import json
import pprint
import sys
from importlib import import_module
from importlib import util as importlib_util
from pathlib import Path


def localized_vars(orig_vars, slice_path):
    """Update (where possible) any references to build-time variables with the best
    guess of the installed location."""
    # The host's sysconfigdata will include references to build-time variables.
    # Update these to refer to the current known install location.
    orig_prefix = orig_vars["prefix"]
    localized_vars = {}
    for key, value in orig_vars.items():
        final = value
        if isinstance(value, str):
            # Replace any reference to the build installation prefix
            final = final.replace(orig_prefix, str(slice_path))
            # Replace any reference to the build-time Framework location
            final = final.replace("-F .", f"-F {slice_path}")
        localized_vars[key] = final

    return localized_vars


def localize_sysconfigdata(sysconfigdata_path, venv_site_packages):
    """Localize a sysconfigdata python module.

    :param support_path: The platform config that contains the sysconfigdata module to
        localize.
    :param venv_site_packages: The site packages folder where the localized
        sysconfigdata module should be output.
    """
    # Import the sysconfigdata module
    spec = importlib_util.spec_from_file_location(
        sysconfigdata_path.stem, sysconfigdata_path
    )
    if spec is None:
        msg = f"Unable to load spec for {sysconfigdata_path}"
        raise ValueError(msg)
    if spec.loader is None:
        msg = f"Spec for {sysconfigdata_path} does not define a loader"
        raise ValueError(msg)
    sysconfigdata = importlib_util.module_from_spec(spec)
    spec.loader.exec_module(sysconfigdata)

    # Write the updated sysconfigdata module into the cross-platform site.
    slice_path = sysconfigdata_path.parent.parent.parent
    with (venv_site_packages / sysconfigdata_path.name).open("w") as f:
        f.write(f"# Generated from {sysconfigdata_path}\n")
        f.write("build_time_vars = ")
        pprint.pprint(
            localized_vars(sysconfigdata.build_time_vars, slice_path),
            stream=f,
            compact=True,
        )

    return sysconfigdata.build_time_vars


def localize_sysconfig_vars(sysconfig_vars_path, venv_site_packages):
    """Localize a sysconfig_vars.json file.

    :param support_path: The platform config that contains the sysconfigdata module to
        localize.
    :param venv_site_packages: The site-packages folder where the localized
        sysconfig_vars.json file should be output.
    :return: The localized sysconfig
    """
    with sysconfig_vars_path.open("rb") as f:
        build_time_vars = json.load(f)

    prefix = sysconfig_vars_path.parent.parent.parent
    sysconfig_vars = localized_vars(build_time_vars, prefix)

    with (venv_site_packages / sysconfig_vars_path.name).open("w") as f:
        json.dump(sysconfig_vars, f, indent=2)

    return sysconfig_vars


def convert_venv(venv_path: Path, sysconfig_path: Path):
    """Convert a virtual environment into a cross-platform environment.

    :param venv_path: The path to the root of the venv.
    :param sysconfig_path: The path to the sysconfig_vars JSON or sysconfigdata Python
        file for the target platform.
    """
    if not venv_path.exists():
        raise ValueError(f"Virtual environment {venv_path} does not exist.")
    if not (venv_path / "bin/python3").exists():
        raise ValueError(f"{venv_path} does not appear to be a virtual environment.")

    # Update path references in the sysconfigdata to reflect local conditions.
    platlibs = list(venv_path.glob("lib/*/site-packages"))
    if len(platlibs) == 0:
        raise ValueError(f"Couldn't find site packages in {venv_path}")
    elif len(platlibs) > 1:
        raise ValueError(f"Found more than one site packages in {venv_path}")

    venv_site_packages_path = platlibs[0]

    if not sysconfig_path.is_file():
        raise ValueError(f"Could not find sysconfig file {sysconfig_path}")

    match sysconfig_path.suffix:
        case ".json":
            sysconfig_vars_path = sysconfig_path
            _, _, _, abiflags, platform, multiarch = sysconfig_vars_path.stem.split("_")

            sysconfigdata_path = (
                sysconfig_vars_path.parent
                / f"_sysconfigdata_{abiflags}_{platform}_{multiarch}.py"
            )
        case ".py":
            sysconfigdata_path = sysconfig_path
            _, _, abiflags, platform, multiarch = sysconfigdata_path.stem.split("_")

            sysconfig_vars_path = (
                sysconfigdata_path.parent
                / f"_sysconfig_vars_{abiflags}_{platform}_{multiarch}.py"
            )
        case _:
            raise ValueError(
                "Don't know how to process sysconfig data "
                f"of type {sysconfig_path.suffix}"
            )

    # Localize the sysconfig data. sysconfigdata *must* exist; sysconfig_vars
    # will only exist on Python 3.14 or newer.
    sysconfig = localize_sysconfigdata(sysconfigdata_path, venv_site_packages_path)
    if sys.version_info[:2] >= (3, 14):
        localize_sysconfig_vars(sysconfig_vars_path, venv_site_packages_path)

    if sysconfig["VERSION"] != venv_site_packages_path.parts[-2][6:]:
        raise ValueError(
            f"target venv is Python {venv_site_packages_path.parts[-2][6:]}; "
            f"sysconfig file is for Python {sysconfig['VERSION']}"
        )

    # Generate the context for the templated cross-target file
    arch, sdk = multiarch.split("-", 1)
    context = {
        "platform": platform,
        "os": platform,  # some platforms use different capitalization here
        "multiarch": multiarch,
        "abiflags": abiflags,
        "arch": arch,
        "sdk": sdk,
    }

    try:
        platform_module = import_module(f"xvenv.platforms.{platform}")
        platform_module.extend_context(context, sysconfig)
    except ImportError:
        raise ValueError(
            f"Don't know how to build a cross-venv file for {platform}"
        ) from None

    cross_multiarch = f"_cross_{platform}_{multiarch.replace('-', '_')}"

    # Render the template for the cross-target file.
    template = (Path(__file__).parent / "_cross_target.py.tmpl").read_text()
    rendered = template.format(**context)
    (venv_site_packages_path / f"{cross_multiarch}.py").write_text(rendered)

    # Write the .pth file that will enable the cross-target modifications
    (venv_site_packages_path / "_cross_venv.pth").write_text(
        f"import {cross_multiarch}\n"
    )

    return f"{context['os']} {multiarch}"



def extend_context(context, sysconfig):
    release = sysconfig["IPHONEOS_DEPLOYMENT_TARGET"]
    is_simulator = context["sdk"] == "iphonesimulator"

    ######################################################################
    context["os"] = "iOS"
    context["release"] = release
    context["platform_version"] = release
    context["machine"] = context["multiarch"]

    # The Darwin kernel version and release are unlikely to be
    # significant, but return realistic values anyway (from an
    # iPhone simulator).
    context["os_sysname"] = "Darwin"
    context["os_nodename"] = "buildmachine.local"
    context["os_release"] = "24.4.0"
    context["os_version"] = (
        "Darwin Kernel Version 24.4.0: Fri Apr 11 18:33:47 PDT 2025; "
        "root:xnu-11417.101.15~117/RELEASE_ARM64_T6000"
    )

    context["platform_extra"] = f"""
    @monkeypatch(platform)
    def ios_ver(system="", release="", model="", is_simulator=False):
        if system == "":
            system = "iOS"
        if release == "":
            release = "{release}"
        if model == "":
            model = "{context["sdk"]}"

        return platform.IOSVersionInfo(system, release, model, {is_simulator})
"""



def extend_context(context, sysconfig):
    # Convert the API level into a release number
    api_level = int(sysconfig["ANDROID_API_LEVEL"])
    if api_level >= 33:
        release = f"{api_level - 20}"
    elif api_level == 32:
        release = "12L"
    elif api_level == 31:
        release = "12"
    elif api_level > 28:
        release = f"{api_level - 19}"
    elif api_level == 27:
        release = "8.1"
    elif api_level == 26:
        release = "8.0"
    elif api_level == 25:
        release = "7.1"
    elif api_level == 24:
        release = "7.0"
    elif api_level == 23:
        release = "6.0"
    elif api_level == 22:
        release = "5.1"
    elif api_level == 21:
        release = "5.0"
    elif api_level == 20:
        release = "4.4W"
    else:
        raise ValueError("xbuild doesn't support API levels lower than 20")

    ######################################################################
    context["os"] = "Android"
    context["release"] = release
    context["platform_version"] = api_level
    context["machine"] = {
        "x86_64": "x86_64",
        "i686": "x86",
        "aarch64": "arm64_v8a",
        "armv7l": "armeabi_v7a",
    }[context["arch"]]

    # The Linux kernel version and release are unlikely to be
    # significant, but return realistic values anyway (from an
    # API level 24 emulator).
    context["os_sysname"] = "Linux"
    context["os_nodename"] = "localhost"
    context["os_release"] = "3.18.91+"
    context["os_version"] = "#1 SMP PREEMPT Tue Jan 9 20:35:43 UTC 2018"

    context["platform_extra"] = f"""
    @monkeypatch(platform)
    def getandroidapilevel() -> int:
        return {api_level}

    @monkeypatch(platform)
    def android_ver(
        release="",
        api_level=0,
        manufacturer="",
        model="",
        device="",
        is_emulator=False
    ):
        if release == "":
            release = "{release}"
        if api_level == 0:
            api_level = {api_level}
        if manufacturer == "":
            manufacturer = "Google"
        if model == "":
            model = "sdk_gphone64"
        if device == "":
            device = "emu64"

        return platform.AndroidVer(
            release, api_level, manufacturer, model, device, True
        )

"""



from __future__ import annotations

import argparse
import json
import os
import sys
from collections.abc import Sequence
from functools import partial
from pathlib import Path

from build import env as _env
from build.__main__ import (
    _cprint,
    _error,
    _format_dep_chain,
    _handle_build_error,
    _max_terminal_width,
    _natural_language_list,
    _setup_cli,
    _styles,
)
from build._types import ConfigSettings, Distribution, StrPath

import xbuild
from xbuild._builder import ProjectXBuilder
from xbuild.env import XBuildIsolatedEnv


def _build(
    isolation: bool,
    srcdir: StrPath,
    outdir: StrPath,
    distribution: Distribution,
    config_settings: ConfigSettings | None,
    skip_dependency_check: bool,
    installer: _env.Installer,
    sysconfig_path: Path | None,
) -> str:
    if isolation:
        return _build_in_isolated_env(
            srcdir,
            outdir,
            distribution,
            config_settings,
            installer,
            sysconfig_path=sysconfig_path,
        )
    else:
        return _build_in_current_env(
            srcdir,
            outdir,
            distribution,
            config_settings,
            skip_dependency_check,
        )


def _build_in_isolated_env(
    srcdir: StrPath,
    outdir: StrPath,
    distribution: Distribution,
    config_settings: ConfigSettings | None,
    installer: _env.Installer,
    sysconfig_path: Path | None,
) -> str:
    with XBuildIsolatedEnv(
        installer=installer,
        sysconfig_path=sysconfig_path,
    ) as env:
        builder = ProjectXBuilder.from_isolated_env(env, srcdir)

        # First install the dependencies for the build
        env.install(builder.build_system_requires, for_target=False)

        # then get the extra required dependencies from the backend
        # (which was installed in the call above :P)
        env.install(
            builder.get_requires_for_build(distribution, config_settings or {}),
            for_target=False,
        )

        # Repeat this process for target dependencies
        env.install(builder.build_system_target_requires)
        # then get the extra required target dependencies from the backend
        env.install(
            builder.get_target_requires_for_build(distribution, config_settings or {})
        )

        # Now run the build
        return builder.build(distribution, outdir, config_settings or {})


def _build_in_current_env(
    srcdir: StrPath,
    outdir: StrPath,
    distribution: Distribution,
    config_settings: ConfigSettings | None,
    skip_dependency_check: bool = False,
) -> str:
    builder = ProjectXBuilder(srcdir)

    if not skip_dependency_check:
        missing = builder.check_dependencies(distribution, config_settings or {})
        if missing:
            dependencies = "".join(
                "\n\t" + dep
                for deps in missing
                for dep in (deps[0], _format_dep_chain(deps[1:]))
                if dep
            )
            _cprint()
            _error(f"Missing build dependencies:{dependencies}")

        missing = builder.check_target_dependencies(distribution, config_settings or {})
        if missing:
            dependencies = "".join(
                "\n\t" + dep
                for deps in missing
                for dep in (deps[0], _format_dep_chain(deps[1:]))
                if dep
            )
            _cprint()
            _error(f"Missing target build dependencies:{dependencies}")

    return builder.build(distribution, outdir, config_settings or {})


def main_parser() -> argparse.ArgumentParser:
    """Construct the main parser."""
    formatter_class = partial(
        argparse.RawDescriptionHelpFormatter, width=min(_max_terminal_width, 127)
    )
    # Workaround for 3.14.0 beta 1, can remove once beta 2 is out
    if sys.version_info >= (3, 14):
        formatter_class = partial(formatter_class, color=True)

    make_parser = partial(
        argparse.ArgumentParser,
        description="A cross-platform build backend for Python",
        # Prevent argparse from taking up the entire width of the terminal window
        # which impedes readability. Also keep the description formatted.
        formatter_class=formatter_class,
    )
    if sys.version_info >= (3, 14):
        make_parser = partial(make_parser, suggest_on_error=True, color=True)

    parser = make_parser()
    parser.add_argument(
        "srcdir",
        type=str,
        nargs="?",
        default=os.getcwd(),
        help="source directory (defaults to current directory)",
    )
    parser.add_argument(
        "--version",
        "-V",
        action="version",
        version=f"xbuild {xbuild.__version__} ({','.join(xbuild.__path__)})",
    )
    parser.add_argument(
        "--verbose",
        "-v",
        dest="verbosity",
        action="count",
        default=0,
        help="increase verbosity",
    )
    ## xbuild doesn't need to produce sdists, which makes --wheel redundant
    # parser.add_argument(
    #     "--sdist",
    #     "-s",
    #     dest="distributions",
    #     action="append_const",
    #     const="sdist",
    #     help="build a source distribution (disables the default behavior)",
    # )
    # parser.add_argument(
    #     "--wheel",
    #     "-w",
    #     dest="distributions",
    #     action="append_const",
    #     const="wheel",
    #     help="build a wheel (disables the default behavior)",
    # )
    parser.add_argument(
        "--outdir",
        "-o",
        type=str,
        help=f"output directory (defaults to {{srcdir}}{os.sep}dist)",
        metavar="PATH",
    )
    parser.add_argument(
        "--skip-dependency-check",
        "-x",
        action="store_true",
        help="do not check that build dependencies are installed",
    )
    env_group = parser.add_mutually_exclusive_group()
    env_group.add_argument(
        "--no-isolation",
        "-n",
        action="store_true",
        help="disable building the project in an isolated virtual environment. "
        "Build dependencies must be installed separately when this option is used",
    )
    env_group.add_argument(
        "--installer",
        choices=_env.INSTALLERS,
        help="Python package installer to use (defaults to pip)",
    )

    # This is only a required argument if the current environment isn't cross-compiling.
    # If/when this project is merged into `build`, the existence of `--sysconfig` as
    # an argument will be the trigger for "this is a cross platform build".
    parser.add_argument(
        "--sysconfig",
        help="The path to a sysconfig_vars JSON file or sysconfigdata Python file",
        required=not getattr(sys, "cross_compiling", False),
    )

    config_group = parser.add_mutually_exclusive_group()
    config_group.add_argument(
        "--config-setting",
        "-C",
        dest="config_settings",
        action="append",
        help=(
            "settings to pass to the backend.  Multiple settings can be "
            "provided. Settings beginning with a hyphen will erroneously "
            "be interpreted as options to build if separated by a space "
            "character; use ``--config-setting=--my-setting -C--my-other-setting``"
        ),
        metavar="KEY[=VALUE]",
    )
    config_group.add_argument(
        "--config-json",
        dest="config_json",
        help=(
            "settings to pass to the backend as a JSON object. This is an "
            "alternative to --config-setting that allows complex nested "
            "structures. Cannot be used together with --config-setting"
        ),
        metavar="JSON_STRING",
    )

    return parser


def main(cli_args: Sequence[str], prog: str | None = None) -> None:
    """Parse the CLI arguments and invoke the build process.

    :param cli_args: CLI arguments
    :param prog: Program name to show in help text
    """
    parser = main_parser()
    if prog:
        parser.prog = prog
    args = parser.parse_args(cli_args)

    _setup_cli(verbosity=args.verbosity)

    config_settings = {}

    # Handle --config-json
    if args.config_json:
        try:
            config_settings = json.loads(args.config_json)
            if not isinstance(config_settings, dict):
                _error(
                    "--config-json must contain a JSON object (dict), "
                    "not a list or primitive value"
                )
        except json.JSONDecodeError as e:
            _error(f"Invalid JSON in --config-json: {e}")

    # Handle --config-setting (original logic)
    elif args.config_settings:
        for arg in args.config_settings:
            setting, _, value = arg.partition("=")
            if setting not in config_settings:
                config_settings[setting] = value
            else:
                if not isinstance(config_settings[setting], list):
                    config_settings[setting] = [config_settings[setting]]

                config_settings[setting].append(value)

    # outdir is relative to srcdir only if omitted.
    outdir = os.path.join(args.srcdir, "dist") if args.outdir is None else args.outdir

    with _handle_build_error():
        sysconfig_path = Path(args.sysconfig) if args.sysconfig else None

        built = [
            _build(
                not args.no_isolation,
                args.srcdir,
                outdir,
                "wheel",
                config_settings,
                args.skip_dependency_check,
                args.installer,
                sysconfig_path=sysconfig_path,
            )
        ]
        artifact_list = _natural_language_list(
            [
                "{underline}{}{reset}{bold}{green}".format(artifact, **_styles.get())
                for artifact in built
            ]
        )
        _cprint("{bold}{green}Successfully built {}{reset}", artifact_list)


def entrypoint() -> None:
    main(sys.argv[1:])


if __name__ == "__main__":  # pragma: no cover
    main(sys.argv[1:], "python -m xbuild")

__all__ = [
    "main",
    "main_parser",
]



def extend_context(context, sysconfig):
    emscripten_version = "4.0.12"
    context["release"] = emscripten_version
    context["platform_version"] = emscripten_version
    context["machine"] = context["arch"]

    context["os_sysname"] = "Emscripten"
    context["os_nodename"] = "emscripten"
    context["os_release"] = emscripten_version
    context["os_version"] = "#1"

    context["platform_extra"] = f"""
    @monkeypatch(platform)
    def libc_ver() -> int:
        return ("emscripten", "{emscripten_version}")
"""



from __future__ import annotations

from collections.abc import Mapping
from typing import Any

from build import _builder, _ctx
from build._builder import ProjectBuilder, _parse_build_system_table
from build._types import ConfigSettings, Distribution
from build._util import check_dependency


###########################################################################
# Patch `build._builder._parse_xbuild_system_table` so that it doesn't break
# in the presence of the `target-requires` setting.
###########################################################################
def parse_xbuild_system_table(pyproject_toml: Mapping[str, Any]) -> Mapping[str, Any]:
    target_requires = pyproject_toml.get("build-system", {}).pop("target-requires", [])

    build_system_table = _parse_build_system_table(pyproject_toml)

    build_system_table["target-requires"] = target_requires

    return build_system_table


_builder._parse_build_system_table = parse_xbuild_system_table


###########################################################################
# A ProjectXBuilder that understands how to handle target requirements
############################################################################
class ProjectXBuilder(ProjectBuilder):
    @property
    def build_system_target_requires(self) -> set[str]:
        """The dependencies defined in the ``pyproject.toml``'s
        ``build-system.target-requires`` field.
        """
        return set(self._build_system["target-requires"])

    def get_target_requires_for_build(
        self,
        distribution: Distribution,
        config_settings: ConfigSettings | None = None,
    ) -> set[str]:
        """Return the dependencies defined by the backend in addition to
        :attr:`build_system_requires` for a given distribution.

        :param distribution: Distribution to get the dependencies of
            (``sdist`` or ``wheel``)
        :param config_settings: Config settings for the build backend
        """
        _ctx.log(f"Getting target build dependencies for {distribution}...")
        hook_name = f"get_target_requires_for_build_{distribution}"
        try:
            get_requires = getattr(self._hook, hook_name)

            with self._handle_backend(hook_name):
                return set(get_requires(config_settings))
        except AttributeError:
            # For now, most build backends won't implement
            # get_target_requires_for_build_wheel
            return set()

    def check_target_dependencies(
        self,
        distribution: Distribution,
        config_settings: ConfigSettings | None = None,
    ) -> set[tuple[str, ...]]:
        """
        Return the dependencies which are not satisfied from the combined set of
        :attr:`build_system_target_requires` and
        :meth:`get_target_requires_for_build` for a given distribution.

        :param distribution: Distribution to check (``sdist`` or ``wheel``)
        :param config_settings: Config settings for the build backend
        :returns: Set of variable-length unmet dependency tuples
        """
        dependencies = self.get_target_requires_for_build(distribution, config_settings)
        dependencies |= self.build_system_target_requires

        return {u for d in dependencies for u in check_dependency(d)}



from __future__ import annotations

import os
import shutil
import sys
import sysconfig
from collections.abc import Collection
from contextlib import contextmanager
from pathlib import Path

from build import _ctx
from build import env as build_env
from build.env import DefaultIsolatedEnv, _PipBackend

from xvenv.convert import convert_venv


###########################################################################
# Patch `build.env._PipBackend` to disable the cross-build environment
# for installs if the install is not for the target.
###########################################################################
@contextmanager
def install_environment(os, for_target):
    """A context manager that temporarily configures the XBUILD_ENV variable.

    :param for_target: If False, disables any active cross-venv by setting
        XBUILD_ENV=off in the environment.
    """
    old_xbuild_env = os.environ.get("XBUILD_ENV", None)
    if not for_target:
        os.environ["XBUILD_ENV"] = "off"

    yield
    if not for_target:
        if old_xbuild_env is None:
            del os.environ["XBUILD_ENV"]
        else:
            os.environ["XBUILD_ENV"] = old_xbuild_env


class _XPipBackend(_PipBackend):
    def install_requirements(
        self,
        requirements: Collection[str],
        for_target: bool = True,
    ) -> None:
        with install_environment(os, for_target=for_target):
            super().install_requirements(requirements)


build_env._PipBackend = _XPipBackend


###########################################################################
# An isolated environment manager that creates cross-plaform installs,
# and can handle both build and target dependency installs.
###########################################################################
class XBuildIsolatedEnv(DefaultIsolatedEnv):
    def __init__(self, *, installer, sysconfig_path):
        if installer == "uv":
            raise RuntimeError("Can't support uv (for now)")

        super().__init__()
        self.sysconfig_path = sysconfig_path

    def __enter__(self) -> XBuildIsolatedEnv:
        super().__enter__()

        # If we're not in a cross-compiling environment, the isolated environment
        # that we create must become a cross-compiling environment. Otherwise,
        # transfer the currently active cross-compilation environment to the
        # isolated environment.
        if not getattr(sys, "cross_compiling", False):
            # We're in a local environment.
            # Make the isolated environment a cross environment.
            if self.sysconfig_path is None:
                raise RuntimeError(
                    "Must specify the location of target platform sysconfig data "
                    "with --sysconfig"
                )

            convert_venv(Path(self._path), self.sysconfig_path)
        else:
            # We're already in a cross environment.
            # Copy any _cross_*.pth or _cross_*.py file, plus the cross-platform
            # sysconfig data to the new environment.
            data_name = sysconfig._get_sysconfigdata_name()
            if sys.version_info < (3, 14):
                vars_files = []
            else:
                vars_files = [sysconfig._get_json_data_name()]

            multiarch = sys.implementation._multiarch.replace("-", "_")
            SRC_SITE_PACKAGES = Path(sysconfig.get_path("platlib"))
            for filename in [
                "_cross_venv.pth",
                f"_cross_{sys.platform}_{multiarch}.py",
                f"{data_name}.py",
            ] + vars_files:
                src = SRC_SITE_PACKAGES / filename
                target = Path(self._path) / src.relative_to(
                    SRC_SITE_PACKAGES.parent.parent.parent
                )
                if not target.exists():
                    shutil.copy(src, target)

        return self

    def install(self, requirements: Collection[str], for_target=True) -> None:
        """Install packages from PEP 508 requirements in the isolated build environment.

        :param requirements: PEP 508 requirement specification to install
        :param for_target: Should the the cross build environment be active? True by
            default; if False, `XBUILD_ENV=off` will be used to install build platform
            binaries, rather than target platform binaries,
        """
        if not requirements:
            return

        for_loc = "target platform" if for_target else "build platform"
        _ctx.log(
            f"Installing packages for {for_loc} in isolated environment:\n"
            + "\n".join(f"- {r}" for r in sorted(requirements))
        )
        self._env_backend.install_requirements(requirements, for_target=for_target)



from __future__ import annotations

import sys
from argparse import ArgumentParser
from collections.abc import Sequence
from pathlib import Path

from build.__main__ import (
    _cprint,
    _error,
)

import xvenv
from xvenv.convert import convert_venv


def main_parser():
    parser = ArgumentParser(
        "xvenv",
        description=(
            "Convert a native virtual environment into a cross-platform "
            "virtual environment."
        ),
    )
    parser.add_argument(
        "--version",
        "-V",
        action="version",
        version=f"xvenv {xvenv.__version__}",
    )
    parser.add_argument(
        "--verbose",
        "-v",
        dest="verbosity",
        action="count",
        default=0,
        help="increase verbosity",
    )
    parser.add_argument(
        "-s",
        "--sysconfig",
        help="The path to a sysconfig_vars JSON file or sysconfigdata Python file",
    )
    parser.add_argument(
        "venv",
        help="The location of a native virtual environment",
    )
    return parser


def main(cli_args: Sequence[str], prog: str | None = None) -> None:
    """Parse the CLI arguments and convert the venv.

    :param cli_args: CLI arguments
    :param prog: Program name to show in help text
    """
    parser = main_parser()
    if prog:
        parser.prog = prog
    args = parser.parse_args(cli_args)

    venv_path = Path(args.venv).resolve()
    sysconfig_path = Path(args.sysconfig).resolve()

    if not venv_path.exists():
        _error(f"Native virtual environment {venv_path} does not exist.")
    else:
        try:
            description = convert_venv(venv_path, sysconfig_path)
        except Exception as e:
            _error(e)
            sys.exit(1)
        else:
            _cprint(
                "{bold}{green}{}{reset}",
                f"{args.venv} is now an {description} cross venv.",
            )


def entrypoint() -> None:
    main(sys.argv[1:])


if __name__ == "__main__":  # pragma: no cover
    main(sys.argv[1:], "python -m xvenv")

__all__ = [
    "main",
    "main_parser",
]
